==> eea.facetednavigation-1.0r1.zip: eea.facetednavigation-1.0r1/eea/facetednavigation/criteria/handler.py <==
from persistent.list import PersistentList
from zope.component import getUtility
from zope.interface import implements
try:
    from zope.annotation.interfaces import IAnnotations
except ImportError:
    #BBB Plone 2.5
    from zope.app.annotation.interfaces import IAnnotations

from zope.app.schema.vocabulary import IVocabularyFactory

from eea.facetednavigation.config import ANNO_CRITERIA
from eea.facetednavigation.widgets.storage import Criterion
from eea.facetednavigation.interfaces import IWidgetsInfo

from interfaces import ICriteria

class Criteria(object):
    """ Handle criteria
    """
    implements(ICriteria)

    def __init__(self, context):
        """ Handle criteria
        """
        self.context = context.getCanonical()
        self.criteria = self._criteria()

    def _criteria(self):
        """ Get criteria from annotations
        """
        anno = IAnnotations(self.context)
        criteria = anno.get(ANNO_CRITERIA, None)
        if criteria is None:
            anno[ANNO_CRITERIA] = PersistentList()
        return anno[ANNO_CRITERIA]

    def _update(self, values):
        anno = IAnnotations(self.context)
        anno[ANNO_CRITERIA] = PersistentList(values)
        self.criteria = anno[ANNO_CRITERIA]
    #
    # Getters
    #
    def newid(self):
        return Criterion().getId()

    def get(self, key, default=None):
        for cid, cvalue in self.items():
            if key == cid:
                return cvalue
        return default

    def keys(self):
        return [criterion.getId() for criterion in self.criteria]

    def values(self):
        return [criterion for criterion in self.criteria]

    def items(self):
        return [(criterion.getId(), criterion) for criterion in self.criteria]
    #
    # Setters
    #
    def add(self, wid, position, section='default'):
        """ Add criterion
        """
        widget = self.widget(wid)
        if not widget:
            raise NameError("Widget type '%s' is undefined" % wid)

        properties = {}
        criteria = self.criteria
        taken_ids = [criterion.getId() for criterion in criteria]
        properties['_taken_ids_'] = taken_ids

        criterion = Criterion(widget=wid, position=position,
                              section=section, **properties)
        criteria.append(criterion)
        return criterion.getId()

    def delete(self, cid):
        """ Delete criteria by given ids
        """
        if not cid:
            raise TypeError('delete takes exactly one argument (0 given)')

        for index, criterion in enumerate(self.criteria):
            criterion_id = criterion.getId()
            if criterion_id == cid:
                self.criteria.pop(index)
                return
        raise KeyError(cid)

    def edit(self, cid, **properties):
        """ Update criterion properties
        """
        criterion = self.get(cid)
        if not criterion:
            raise KeyError(cid)

        criterion.update(**properties)
        self.criteria._p_changed = 1
    #
    # Position
    #
    def up(self, cid):
        """ Move criterion up
        """
        insert = None
        for index, criterion in enumerate(self.criteria):
            if criterion.getId() == cid:
                insert = self.criteria.pop(index)
                break
        if not insert:
            raise KeyError(cid)

        if index > 0:
            index -= 1
        self.criteria.insert(index, insert)

    def down(self, cid):
        """ Move criterion down
        """
        insert = None
        for index, criterion in enumerate(self.criteria):
            if criterion.getId() == cid:
                insert = self.criteria.pop(index)
                break
        if not insert:
            raise KeyError(cid)

        index += 1
        self.criteria.insert(index, insert)

    def position(self, **kwargs):
        """ Update criteria position from given lists
        """
        voc = getUtility(IVocabularyFactory,
                         'eea.faceted.vocabularies.WidgetPositions')
        positions = ((term.value, kwargs.get(term.value, []))
                     for term in voc(self.context))

        res = []
        for position, cids in positions:
            for cid in cids:
                criterion = self.get(cid)
                criterion.update(position=position)
                res.append(criterion)
        self._update(res)
    #
    # Utils
    #
    def widget(self, wid=None, cid=None):
        """ Return widget by given wid or from criterion by given cid
        """
        if not wid:
            criterion = self.get(cid)
            if not criterion:
                raise KeyError(cid)
            wid = criterion.get('widget')
        info = getUtility(IWidgetsInfo)
        return info.widgets.get(wid, None)

==> eea.facetednavigation-3.6.zip: eea.facetednavigation-3.6/eea/facetednavigation/criteria/handler.py <==
from persistent.list import PersistentList
from zope.component import getUtility
from zope.interface import implements
try:
    from zope.annotation.interfaces import IAnnotations
except ImportError:
    #BBB Plone 2.5
    from zope.app.annotation.interfaces import IAnnotations

from zope.app.schema.vocabulary import IVocabularyFactory

from eea.facetednavigation.config import ANNO_CRITERIA
from eea.facetednavigation.widgets.storage import Criterion
from eea.facetednavigation.interfaces import IWidgetsInfo

from interfaces import ICriteria

class Criteria(object):
    """ Handle criteria
    """
    implements(ICriteria)

    def __init__(self, context):
        """ Handle criteria
        """
        # LinguaPlone support
        canonical = getattr(context, 'getCanonical', None)
        if canonical:
            self.context = canonical()
        else:
            self.context = context
        self.criteria = self._criteria()

    def _criteria(self):
        """ Get criteria from annotations
        """
        anno = IAnnotations(self.context)
        criteria = anno.get(ANNO_CRITERIA, None)
        if criteria is None:
            anno[ANNO_CRITERIA] = PersistentList()
        return anno[ANNO_CRITERIA]

    def _update(self, values):
        anno = IAnnotations(self.context)
        anno[ANNO_CRITERIA] = PersistentList(values)
        self.criteria = anno[ANNO_CRITERIA]
    #
    # Getters
    #
    def newid(self):
        return Criterion().getId()

    def get(self, key, default=None):
        for cid, cvalue in self.items():
            if key == cid:
                return cvalue
        return default

    def keys(self):
        return [criterion.getId() for criterion in self.criteria]

    def values(self):
        return [criterion for criterion in self.criteria]

    def items(self):
        return [(criterion.getId(), criterion) for criterion in self.criteria]
    #
    # Setters
    #
    def add(self, wid, position, section='default', **kwargs):
        """ Add criterion
        """
        widget = self.widget(wid)
        if not widget:
            raise NameError("Widget type '%s' is undefined" % wid)

        properties = {}
        criteria = self.criteria
        taken_ids = [criterion.getId() for criterion in criteria]
        properties['_taken_ids_'] = taken_ids
        properties.update(kwargs)

        criterion = Criterion(widget=wid, position=position,
                              section=section, **properties)
        criteria.append(criterion)
        return criterion.getId()

    def delete(self, cid):
        """ Delete criteria by given ids
        """
        if not cid:
            raise TypeError('delete takes exactly one argument (0 given)')

        for index, criterion in enumerate(self.criteria):
            criterion_id = criterion.getId()
            if criterion_id == cid:
                self.criteria.pop(index)
                return
        raise KeyError(cid)

    def edit(self, cid, **properties):
        """ Update criterion properties
        """
        criterion = self.get(cid)
        if not criterion:
            raise KeyError(cid)

        criterion.update(**properties)
        self.criteria._p_changed = 1
    #
    # Position
    #
    def up(self, cid):
        """ Move criterion up
        """
        insert = None
        for index, criterion in enumerate(self.criteria):
            if criterion.getId() == cid:
                insert = self.criteria.pop(index)
                break
        if not insert:
            raise KeyError(cid)

        if index > 0:
            index -= 1
        self.criteria.insert(index, insert)

    def down(self, cid):
        """ Move criterion down
        """
        insert = None
        for index, criterion in enumerate(self.criteria):
            if criterion.getId() == cid:
                insert = self.criteria.pop(index)
                break
        if not insert:
            raise KeyError(cid)

        index += 1
        self.criteria.insert(index, insert)

    def position(self, **kwargs):
        """ Update criteria position from given lists
        """
        voc = getUtility(IVocabularyFactory,
                         'eea.faceted.vocabularies.WidgetPositions')
        positions = ((term.value, kwargs.get(term.value, []))
                     for term in voc(self.context))

        res = []
        for position, cids in positions:
            for cid in cids:
                criterion = self.get(cid)
                criterion.update(position=position)
                res.append(criterion)
        self._update(res)
    #
    # Utils
    #
    def widget(self, wid=None, cid=None):
        """ Return widget by given wid or from criterion by given cid
        """
        if not wid:
            criterion = self.get(cid)
            if not criterion:
                raise KeyError(cid)
            wid = criterion.get('widget')
        info = getUtility(IWidgetsInfo)
        return info.widgets.get(wid, None)
